/*******************************************************************************
 * Copyright (c) 2015 Texas Instruments and others All rights reserved. This
 * program and the accompanying materials are made available under the terms of
 * the Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Gingrich, Paul - Initial API and implementation
 ******************************************************************************/
var gc = gc || {};
gc.databind = gc.databind || {};

(function() // closure for private static methods and data.
{
    var numPacketsReceived = 0;
    
    var UsbHidModel = function()
    {
        gc.databind.AbstractStreamingDataModel.call(this);
    };

    UsbHidModel.prototype = new gc.databind.AbstractStreamingDataModel('usbhid');

    var usbHID;
    
    UsbHidModel.prototype.sendValue = function(value, progress)
    {
        if (this._encoder)
        {
            var packet = this._encoder.encode(value);
            if (usbHID)
            {
                usbHID.serialSend(packet);
            }
            else
            {
                console.log('Discarded tx uart data: ' + packet);
            }
        }
    };

    var usbHidPortInstance = new UsbHidModel();
    var callbackForDataValidation;

    gc.databind.UsbHidModel =
    {
        usbHidPortRecievedHandler :
        {
            text: function(message, buffer)
            {
                if (! usbHidPortInstance._decoder){
                    gc.databind.registerCustomCodec('JsonCodec',gc.databind.internal.JsonCodec);
                    usbHidPortInstance._decoder = new gc.databind.internal.JsonCodec();
                }
                var decoder = usbHidPortInstance._decoder;

                if (decoder)
                {
                    decoder.decode(buffer, usbHidPortInstance);

                    var isConnected = true;
                    if (decoder.isConnected) {
                        isConnected = decoder.isConnected();
                    }
                    usbHidPortInstance.setConnectedState(isConnected);
                    if (callbackForDataValidation && isConnected)
                    {
                        callbackForDataValidation();
                        callbackForDataValidation = undefined;
                    }
                }
                else
                {
                    console.error('UsbHidModel: data received before decoder initialized');
                }
            }
        },
        connect : function(dataValidationCallback, rxPacketDataFormat)
        {
            usbHidPortInstance._decoder = usbHidPortInstance.createCodec(rxPacketDataFormat);
            callbackForDataValidation = dataValidationCallback;
        },
        disconnect : function()
        {
            callbackForDataValidation = undefined;
            usbHidPortInstance._encoder = undefined;
            usbHidPortInstance._decoder = undefined;
            usbHidPortInstance.setConnectedState(false);
        },
        setUsbHIDInstance : function(usbHidServiceInstance, txPacketDataFormat)
        {
            usbHID = usbHidServiceInstance;
            usbHidPortInstance._encoder = usbHidPortInstance.createCodec(txPacketDataFormat);
        }
    };

    // register serial model with bindingRegistry
    gc.databind.registry.registerModel(new gc.databind.StorageProviderBindFactoryWrapper(usbHidPortInstance), true);

}());
